# newton-sysid2015

This code was downloaded from < http://users.isy.liu.se/en/rt/johda87/ > and contains the code used to produce the results in the paper

* M. Kok, J. Dahlin, T. B. Schön and A. Wills, *Newton-based maximum likelihood estimation in nonlinear state space models*. Proceedings of the 17th IFAC Symposium on System Identification, Beijing, China, October 2015. 

The papers are available as a preprint from < http://arxiv.org/pdf/1502.03655v2 > and < http://users.isy.liu.se/en/rt/johda87/ >.

Requirements
--------------
The program is written in Python 2.7 and makes use of NumPy 1.7.1, SciPy 0.12.0, Matplotlib 1.2.1, Pandas. Please have these packages installed, on Ubuntu they can be installed using "sudo pip install --upgrade *package-name* ".

Included files
--------------
**RUNME-example1.py**
Recreates a subset of the first numerical illustration in the paper. Executes the proposed algorithm for Newton optimisation with particle smoothing (Algorithm 3 in the paper) using the fixed-lag (FL) particle smoother and the fast forward-filtering backward-simulation (FFBSi) particle smoother. The optimisation algorithm is run for 100 iterations and the trace plot of the parameter estimates are displayed after the run. Note that the parameter inference is only made for dataset 1, whereas in the paper the same procedure is repeated for all the 100 data sets. The results and plots from a single run is attached in the results folder.

**RUNME-example2.py**
Recreates a subset of the second numerical illustration in the paper. All the details are the same as for example 1.

Supporting files
--------------
**models/newton_sysid2015_example1.py**
Defines the state space model for example 1 that we use together with the expressions required to simulate from the model and estimate the gradient. 

**models/newton_sysid2015_example2.py**
Defines the state space model for example 1 that we use together with the expressions required to simulate from the model and estimate the gradient. 

**models/models_helpers.py**
Subroutines for data generation and for importing data.

**para/ml_opt.py**
The main routine for Newton optimiasation.

**para/ml_helpers.py**
Subroutines for exporting the data generated by the optimisation algorithm.

**results/**
Data and plots from running the two RUNME-files.

**state/smc.py**
The main wrapper for particle filtering and smoothing.

**state/smc_filter.py**
The routines for particle filtering to estimate the log-likelihood and generate the particle system.

**state/smc_smoothing.py**
The routines for particle smoothing to estimate the gradient of the log-likelihood.

**state/smc_helpers.py**
Resampling routines and the estimation of the Hessian of the log-likelihood via the Segal-Weinstein estimator.
